/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.spi;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.text.Collator;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.observation.ObservationManager;
import it.tidalwave.observation.bluebill.ObservationClipboard;
import it.tidalwave.mobile.preferences.PreferencesAdapter;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.taxonomy.mobile.Taxonomy;
import it.tidalwave.bluebill.taxonomy.mobile.TaxonomyManager;
import it.tidalwave.bluebill.taxonomy.mobile.impl.SimpleTaxonomyManager;
import it.tidalwave.bluebill.mobile.taxonomy.text.FastNameMatcher;
import it.tidalwave.bluebill.mobile.taxonomy.text.ExactSubstringNameMatcher;
import it.tidalwave.bluebill.mobile.taxonomy.text.CamelCaseMatcher;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonHistory;
import it.tidalwave.bluebill.mobile.preferences.NameMatcher;
import it.tidalwave.bluebill.mobile.preferences.TaxonomyPreferences;
import it.tidalwave.bluebill.taxonomy.mobile.TaxonomyFinder;
import it.tidalwave.util.Id;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class TaxonomyPreferencesSupport implements TaxonomyPreferences // FIXME: rename to DefaultTaxonomyPreferences
  {
    public static final String PREF_RENDER_SCIENTIFIC_NAMES = "renderScientificNames";
    public static final String PREF_PRIMARY_LOCALE = "taxonomyPrimaryLocale";
    public static final String PREF_SECONDARY_LOCALE = "taxonomySecondaryLocale";
    public static final String PREF_NAME_MATCHER = "nameMatcher/3";
    public static final String PREF_TAXONOMY = "taxonomySchema";
   
    private static final String DEFAULT_NAME_MATCHER = "FastNameMatcherWithInitial";

    private static final String DEFAULT_TAXONOMY_ID = "http://bluebill.tidalwave.it/taxonomy/EBNItalia/2003";
    
    @CheckForNull
    protected Taxonomy taxonomy;

    protected final List<Locale> taxonLocales = new ArrayList<Locale>();

    protected Locale primaryTaxonLocale = Locale.ENGLISH;

    @Getter
    protected NameMatcher nameMatcher;

    protected Collator taxonCollator = Collator.getInstance();

    protected final Map<String, NameMatcher> matcherMapByKey = new HashMap<String, NameMatcher>();

    protected final Provider<PreferencesAdapter> preferencesAdapter = Locator.createProviderFor(PreferencesAdapter.class);

    private final Provider<ObservationClipboard> observationClipboard = Locator.createProviderFor(ObservationClipboard.class);
    
    private final Provider<TaxonHistory> taxonHistory = Locator.createProviderFor(TaxonHistory.class);
    
    private final Provider<ObservationManager> observationManager = Locator.createProviderFor(ObservationManager.class);
        
    public int taxonomyLoadCount = 0;
    
    private final TaxonomyManager taxonomyManager;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Getter
    private final Comparator<Taxon> taxonComparator = new Comparator<Taxon>()
      {
        public int compare (final @Nonnull Taxon taxon1, @Nonnull final Taxon taxon2)
          {
            return taxonCollator.compare(getDisplayName(taxon1), getDisplayName(taxon2));
          }

        @Nonnull
        private String getDisplayName (final @Nonnull Taxon taxon)
          {
            try
              {
                return taxon.getDisplayName(primaryTaxonLocale);
              }
            catch (NotFoundException e)
              {
                // We prepend some zeds to scientific names, so if they are compared against common names,
                // they go to the bottom of the list.
                // FIXME: in some locales, Z is not the last letter!
                return "ZZZZZZZ" + taxon.getScientificName();
              }
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public TaxonomyPreferencesSupport()
      {
        matcherMapByKey.put("FastNameMatcher", new FastNameMatcher(false));
        matcherMapByKey.put("FastNameMatcherWithInitial", new FastNameMatcher(true));
        matcherMapByKey.put("CamelCaseMatcher", new CamelCaseMatcher());
        matcherMapByKey.put("ExactSubstringNameMatcher", new ExactSubstringNameMatcher());
        
        taxonomyManager = createTaxonomyFactory();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Id getSelectedTaxononyId()
      {
        return new Id(preferencesAdapter.get().getString(PREF_TAXONOMY, DEFAULT_TAXONOMY_ID));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public boolean isScientificNamesRenderingEnabled()
      {
        return preferencesAdapter.get().getBoolean(PREF_RENDER_SCIENTIFIC_NAMES, true);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public TaxonomyFinder findTaxonomies()
      {
        return taxonomyManager.findTaxonomies();  
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public synchronized Taxonomy getTaxonomy()
      {
        if (taxonomy == null)
          {
            try
              {
                taxonomyLoadCount++;
                taxonomy = loadTaxonomy();
              }
            catch (Exception e)
              {
                throw new RuntimeException(e);
              }
          }

        return taxonomy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void setTaxonomy (final @Nonnull Taxonomy taxonomy) 
      {
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void resetTaxonomy()
      {
        log.info("resetTaxonomy()");
        taxonomy = null;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Taxonomy reloadTaxonomy()
      {
        log.info("reloadTaxonomy()");
        taxonomy = null;
        observationClipboard.get().clear();
        taxonHistory.get().clearTaxonHistory();
        observationManager.get().findOrCreateObservationSetById(null).clear();
        
        return getTaxonomy();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public List<Locale> getTaxonomyLocales()
      {
        return Collections.unmodifiableList(taxonLocales);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void reset()
      {
        computeLocales();
        setNameMatcher();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Taxonomy loadTaxonomy()
      throws Exception
      {
        return loadTaxonomy(getSelectedTaxononyId());
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Taxonomy loadTaxonomy (final @Nonnull Id taxonomyId)
      throws Exception
      {
        log.info("loadTaxonomy({})", taxonomyId);
        final long time = System.currentTimeMillis();
        final Taxonomy taxonomy = taxonomyManager.findTaxonomies().withId(taxonomyId).result();
        log.info(">>>> taxonomy loaded in {} msecs", System.currentTimeMillis() - time);
        return taxonomy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void computeLocales()
      {
        log.info("computeLocales() - default is {}", Locale.getDefault());
        primaryTaxonLocale = getLocale(PREF_PRIMARY_LOCALE, Locale.getDefault());
        final Locale secondaryTaxonLocale = getLocale(PREF_SECONDARY_LOCALE, primaryTaxonLocale);
        taxonLocales.clear();
        taxonLocales.add(primaryTaxonLocale);

        if (!primaryTaxonLocale.equals(secondaryTaxonLocale))
          {
            taxonLocales.add(secondaryTaxonLocale);
          }

        log.info(">>>> locales: {}, primary: {}", taxonLocales, primaryTaxonLocale);
        taxonCollator = Collator.getInstance(primaryTaxonLocale);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected Locale getLocale (final @Nonnull String key, final @Nonnull Locale defaultLocale)
      {
        final String localeCode = preferencesAdapter.get().getString(key, "");
        return localeCode.equals("") ? defaultLocale : new Locale(localeCode);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected void setNameMatcher()
      {
        final String key = preferencesAdapter.get().getString(PREF_NAME_MATCHER, DEFAULT_NAME_MATCHER);
        nameMatcher = matcherMapByKey.get(key);

        if (nameMatcher == null) // defensive
          {
            log.warn("Cannot set name matcher {} - available: {} - using a default", key, matcherMapByKey.keySet());
            nameMatcher = new FastNameMatcher(true);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected TaxonomyManager createTaxonomyFactory() 
      {
        return new SimpleTaxonomyManager();
      }
  }