/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.ui.spi;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ExecutorService;
import javax.swing.Action;
import it.tidalwave.util.RoleFactory;
import it.tidalwave.util.Task;
import it.tidalwave.util.ui.FlowController;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.role.ui.spi.SingleActionProvider;
import it.tidalwave.role.ui.spi.DefaultPresentationModel;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.capabilitiesprovider.ThreadLookupBinder;
import it.tidalwave.bluebill.mobile.preferences.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.role.util.RoleRegister;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonHistory;
import it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonPickerView;
import it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonPickerViewController;
import it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonomyFooterController;
import it.tidalwave.bluebill.mobile.taxonomy.impl.BrowseByRankAction;
import it.tidalwave.bluebill.mobile.taxonomy.impl.SelectTaxonAction;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon;
import it.tidalwave.bluebill.taxonomy.mobile.Taxon.Rank;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import static it.tidalwave.util.ui.UserNotification.notification;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor @Slf4j
public class DefaultTaxonPickerViewController implements TaxonPickerViewController
  {
    private static final Class<DefaultTaxonPickerViewController> _ = DefaultTaxonPickerViewController.class;

    @Nonnull
    private final TaxonPickerView view;
    
    @Nonnull
    private final FlowController flowController;
    
    /* package */ final DefaultPresentationModel taxaPM = new DefaultPresentationModel();

    private final TaxonomyPreferences preferences = Locator.find(TaxonomyPreferences.class);

    private final TaxonHistory taxonHistory = Locator.find(TaxonHistory.class);

    private final TaxonomyFooterController taxonomyFooterController = new TaxonomyFooterController();
    
    protected final RoleRegister roleRegister = new RoleRegister();
    
    @Nonnull
    private final ExecutorService executorService = Locator.find(ExecutorService.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final RoleFactory<PresentationModel> actionProviderRoleFactory = new RoleFactory<PresentationModel> () 
      {
        @Nonnull
        public Object createRoleFor (final @Nonnull PresentationModel taxonPM) 
          {
            return new SingleActionProvider(new SelectTaxonAction(taxonPM, flowController));
          }
      };
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
      {
        roleRegister.registerRoleFactory(actionProviderRoleFactory).forClass(PresentationModel.class);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Action getBrowseByRankAction (final @Nonnull Rank rank) 
      {
        return new BrowseByRankAction(rank, flowController);
      }
        
    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void initialize() 
      {
        log.info("initialize()");
        view.lock(notification().withText(_, "preparingData")); 
        view.showTaxonomyName(taxonomyFooterController.getTaxonomyDisplayName());
        view.showTaxonomyLanguages(taxonomyFooterController.getTaxonLanguages());
        
        executorService.submit(new Runnable()
          {
            @Override
            public void run()
              {         
                populateView();
              }
          });
      }
    
    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void clearTaxonHistory()
      {
        log.info("clearTaxonHistory()");
        taxonHistory.clearTaxonHistory();
        taxaPM.clear();
        view.populate(taxaPM);
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    protected void populateView()
      {
        log.info("populateView()");
        final List<Taxon> taxa = taxonHistory.getRecentTaxa();
        final long time = System.currentTimeMillis();
        Collections.sort(taxa, preferences.getTaxonComparator());
        log.debug(">>>> {} taxa sorted in {} msec", taxa.size(), System.currentTimeMillis() - time);
        
        taxaPM.clear();
        
        ThreadLookupBinder.with(roleRegister).run(new Task<Void, RuntimeException>() 
          {
            @Override
            public Void run()
              {
                for (final Taxon taxon : taxa)
                  {
                    taxaPM.addChild(new DefaultPresentationModel(taxon));                
                  }

                return null;
              }
          });
        
        view.populate(taxaPM);
        view.unlock();
      }
  }
