/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.taxonomy.ui.spi;

import javax.annotation.Nonnull;
import javax.inject.Provider;
import java.util.concurrent.ExecutorService;
import it.tidalwave.util.Id;
import it.tidalwave.util.RoleFactory;
import it.tidalwave.util.Task;
import it.tidalwave.util.ui.FlowController;
import it.tidalwave.util.ui.UserNotificationWithFeedback.Feedback;
import it.tidalwave.role.Identifiable;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.role.ui.spi.DefaultPresentationModel;
import it.tidalwave.role.ui.spi.SingleActionProvider;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.capabilitiesprovider.ThreadLookupBinder;
import it.tidalwave.observation.Observation;
import it.tidalwave.observation.ObservationManager;
import it.tidalwave.observation.ObservationSet;
import it.tidalwave.observation.bluebill.ObservationClipboard;
import it.tidalwave.bluebill.taxonomy.mobile.Taxonomy;
import it.tidalwave.bluebill.mobile.preferences.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.role.util.RoleRegister;
import it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonomyPickerView;
import it.tidalwave.bluebill.mobile.taxonomy.ui.TaxonomyPickerViewController;
import it.tidalwave.bluebill.mobile.taxonomy.impl.SelectTaxonomyAction;
import lombok.RequiredArgsConstructor;
import static it.tidalwave.util.ui.UserNotificationWithFeedback.notificationWithFeedback;

/***********************************************************************************************************************
 *
 * @stereotype Controller
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public class DefaultTaxonomyPickerViewController implements TaxonomyPickerViewController
  {
    private static final Class<DefaultTaxonomyPickerViewController> _ = DefaultTaxonomyPickerViewController.class;
    
    @Nonnull
    protected final TaxonomyPickerView view;
    
    @Nonnull 
    protected final FlowController flowController;

    @Nonnull
    private final Provider<ObservationManager> observationManager = Locator.createProviderFor(ObservationManager.class);

    @Nonnull
    private final Provider<ObservationClipboard> observationClipboard = Locator.createProviderFor(ObservationClipboard.class);
    
    @Nonnull
    private final Provider<TaxonomyPreferences> taxonomyPreferences = Locator.createProviderFor(TaxonomyPreferences.class);
    
    @Nonnull
    private final ExecutorService executorService = Locator.find(ExecutorService.class);

    protected final DefaultPresentationModel pmTaxonomies = new DefaultPresentationModel();
    
    protected final RoleRegister roleRegister = new RoleRegister();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final RoleFactory<PresentationModel> actionProviderRoleFactory = new RoleFactory<PresentationModel> () 
      {
        @Nonnull
        public Object createRoleFor (final @Nonnull PresentationModel taxonomyPM) 
          {
            return new SingleActionProvider(new SelectTaxonomyAction(taxonomyPM, flowController));
          }
      };
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
      {
        roleRegister.registerRoleFactory(actionProviderRoleFactory).forClass(PresentationModel.class);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    public void initialize()
      {
        executorService.submit(new Runnable()
          {
            @Override
            public void run()
              {
                ThreadLookupBinder.with(roleRegister).run(new Task<Void, RuntimeException>() 
                  {
                    @Override
                    public Void run()
                      {
                        int index = 0;
                        int selectedIndex = 0;
                        
                        final Id selectedTaxonomyId = taxonomyPreferences.get().getTaxonomy().as(Identifiable.class).getId();
                        
                        for (final Taxonomy taxonomy : taxonomyPreferences.get().findTaxonomies().results())
                          {
                            pmTaxonomies.addChild(new DefaultPresentationModel(taxonomy)); 
                            
                            if (taxonomy.as(Identifiable.class).getId().equals(selectedTaxonomyId))
                              {
                                selectedIndex = index;
                              }
                            
                            index++;
                          }
                        
                        view.populate(pmTaxonomies);
                        view.selectIndex(selectedIndex);
                        checkWhetherTheTaxonomyCanBeChanged();
                        
                        return null;
                      }
                  });
              }
          });
      }

    /*******************************************************************************************************************
     *
     * Checks whether the taxonomy can be changed right now. A taxonomy cannot be changed if there are stored 
     * observations or a new observation is being inserted. When it's unfeasible to change the taxonomy, this method
     * will terminate the UI.
     *
     ******************************************************************************************************************/
    private void checkWhetherTheTaxonomyCanBeChanged()
      {
        final ObservationSet observationSet = observationManager.get().findOrCreateObservationSetById(null);
        final boolean observationsInserted = observationSet.find(Observation.class).count() > 0;
        final boolean insertInProgress = observationClipboard.get().getBuilder().getItemCount() > 0;

        if (observationsInserted || insertInProgress)
          {
            view.notifyThatTheTaxonomyCannotBeChanged(notificationWithFeedback().withCaption(_, "cantChangeChecklist_title")
                                                                                .withText(_, "cantChangeChecklist_message")
                                                                                .withFeedback(new Feedback()
              {
                @Override
                public void onConfirm()
                  {
                    flowController.finish();
                  }
              }));
          }
      }
  }
